/*
 * CCUSBDMX.H -- include file for Peperoni-Light's CCUSBDMX.DLL
 *   to communicate with Cooper Controls DMX Dongle usb to dmx512
 *   interfaces.
 *
 * Copyright (C) 2004-2007 by Jan Menzel
 * All rights reserved 
 */

#ifndef CCUSBDMX_H
#define CCUSBDMX_H

// define the dll space this file is used in
#ifdef CCUSBDMX_DLL_EXPORT
#define CCUSBDMX_TYPE __declspec(dllexport) __stdcall
#else
#define CCUSBDMX_TYPE __declspec(dllimport) __stdcall
#endif

/*
 * DLL version
 */
static USHORT CCUSBDMX_DLL_VERSION = 0x0403;

/*
 * MACRO to verify dll version
 */
#define CCUSBDMX_DLL_VERSION_CHECK() (ccusbdmx_version() >= CCUSBDMX_DLL_VERSION)

/* *************************************
 * functions defined in the ccusbdmx.dll *
 ***************************************/

/*
 *   ccusbdmx_version(): returns the version number (16bit, 4 digits BCD)
 * Current version is CCUSBDMX_DLL_VERSION. Use the Macro 
 * CCUSBDMX_DLL_VERSION_CHECK() compare dll's and header files version.
 */
USHORT	CCUSBDMX_TYPE	ccusbdmx_version();

/*
 *   ccusbdmx_open(): open device number <device>, where 0 is the first
 * and unlimit devices are supported. The function returnes 
 * STATUS_INVALID_HANDLE_VALUE if <device> is not supported. Use the
 * returned handle to access the device later on. One device can be
 * opened an unlimited number of times.
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_open(USHORT device, PHANDLE h);

/*
 *   ccusbdmx_close(): close the device identified by the given handle.
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_close(HANDLE h);


/*
 *   ccusbdmx_device_id(): read the device id of the device
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_device_id(HANDLE h, PUSHORT pid);

/*
 *   ccusbdmx_is_XXX(): identify the device identified by the given handle.
 * Each function returns TRUE if the device matches.
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_is_oneuniverse(HANDLE h);
BOOL	CCUSBDMX_TYPE	ccusbdmx_is_twouniverse(HANDLE h);

/*
 *   ccusbdmx_product_get(): read the product string from the device.
 * size specifies the maximum size of the buffer pointed to by <string> 
 * (unit bytes).
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_product_get(HANDLE h, PWCHAR string, USHORT size);

/*
 *   ccusbdmx_serial_get(): read the serial string from the device.
 * size specifies the maximum size of the buffer pointed to by <string> 
 * (unit bytes).
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_serial_get(HANDLE h, PWCHAR string, USHORT size);

/*
 *   ccusbdmx_device_version(): Read the the device version of a device.
 * the device version is one of the values within the USBs configuration
 * descriptor (BcdDevice). pversion is only valid if the function returns
 * TRUE.
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_device_version(HANDLE h, PUSHORT pversion);

/*
 *   CCUSBDMX_TX(): transmitt a frame using the new protocol on bulk endpoints
 *
 * INPUTs:	h			- handle to the device, returned by ccusbdmx_open()
 *			universe	- addressed universe
 *			slots		- number of bytes to be transmitted, as well as sizeof(buffer)
 *						  for DMX512: buffer[0] == startcode, slots <= 513
 *			buffer		- data to be transmitted,  !!! sizeof(buffer) >= slots !!!
 *			config		- configuration of the transmitter, see below for possible values
 *			time		- time value in s, depending on config, either timeout or delay
 *			time_break	- break time in s (can be zero, to not transmitt a break)
 *			time_mab	- Mark-after-Break time (can be zero)
 * OUTPUTs:	ptimestamp	- timestamp of this frame in ms, does overrun
 *			pstatus		- status of this transmission, see below for possible values
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_tx(IN HANDLE h, IN UCHAR universe, IN USHORT slots, 
									IN LPCVOID buffer, IN UCHAR config, IN FLOAT time, 
									IN FLOAT time_break, IN FLOAT time_mab, 
									OUT PUSHORT ptimestamp, OUT PUCHAR pstatus);
/*
 * values of config (to be ored together)
 */
#define CCUSBDMX_BULK_CONFIG_DELAY	(0x01)	// delay new frame by time with respect to previous
#define CCUSBDMX_BULK_CONFIG_BLOCK	(0x02)	// block while frame is not transmitting (timeout given by time)
#define CCUSBDMX_BULK_CONFIG_RX		(0x04)	// switch to RX after having transmitted this frame
#define CCUSBDMX_BULK_CONFIG_NORETX	(0x08)	// do not retransmit this frame
#define CCUSBDMX_BULK_CONFIG_TXIRQ	(0x40)	// send data using transmitter IRQ instead of timer

/*
 *   CCUSBDMX_RX(): receive a frame using the new protocol on bulk endpoints
 *
 * INPUTs:	h			- handle to the device, returned by ccusbdmx_open()
 *			universe	- addressed universe
 *			slots_set	- number of bytes to receive, as well as sizeof(buffer)
 *						  for DMX512: buffer[0] == startcode, slots_set <= 513
 *			buffer		- data to be transmitted,  !!! sizeof(buffer) >= slots !!!
 *			timeout		- timeout for receiving the total frame in s,
 *			timeout_rx	- timeout between two slots used to detect premature end of frames
 * OUTPUTs:	pslots_get	- number of slots actually received, *pslots_get <= slots_set
 *          ptimestamp	- timestamp of this frame in ms, does overrun
 *			pstatus		- status of the reception, see below for possible values
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_rx(IN HANDLE h, IN UCHAR universe, IN USHORT slots_set, 
									IN LPVOID buffer, IN FLOAT timeout, IN FLOAT timeout_rx,
									OUT PUSHORT pslots_get, OUT PUSHORT ptimestamp, OUT PUCHAR pstatus);

/*
 * values of *pstatus
 */
#define CCUSBDMX_BULK_STATUS_OK					(0x00)
#define CCUSBDMX_BULK_STATUS_TIMEOUT			(0x01)	// request timed out	
#define CCUSBDMX_BULK_STATUS_TX_START_FAILED	(0x02)	// delayed start failed
#define CCUSBDMX_BULK_STATUS_UNIVERSE_WRONG		(0x03)	// wrong universe addressed\tabularnewline
#define CCUSBDMX_BULK_STATUS_RX_OLD_FRAME		(0x10)	// old frame not read
#define CCUSBDMX_BULK_STATUS_RX_TIMEOUT			(0x20)	// receiver finished with timeout (ored with others)
#define CCUSBDMX_BULK_STATUS_RX_NO_BREAK		(0x40)	// frame without break received (ored with others)
#define CCUSBDMX_BULK_STATUS_RX_FRAMEERROR		(0x80)	// frame finished with frame error (ored with others)

/*
 * macro to check, it the return status is ok
 */
#define CCUSBDMX_BULK_STATUS_IS_OK(s) (s == CCUSBDMX_BULK_STATUS_OK)

/*
 *   ccusbdmx_id_led_XXX(): get/set the "id-led", the way the TX-led is handled:
 * special value: see below
 * other:         the blue led blinks the given number of times and then pauses
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_id_led_set(HANDLE h, UCHAR id);
BOOL	CCUSBDMX_TYPE	ccusbdmx_id_led_get(HANDLE h, PUCHAR id);

/*
 * special values of id
 */
#define CCUSBDMX_ID_LED_USB		(0xff)	// display the USB status: blink with 2Hz on USB transactions
#define CCUSBDMX_ID_LED_USB_RX	(0xfe)	// display USB and receiver status. the LED blinks red if not valid dmx signal in received

/*
 *   ccusbdmx_eeprom_[set|get](): write/read data to/from eeprom of the interface.
 * size specifies the amount of bytes to transfer from buffer/interface to interface/buffer.
 */
BOOL	CCUSBDMX_TYPE	ccusbdmx_eeprom_set(HANDLE h, LPCVOID buffer, USHORT size);
BOOL	CCUSBDMX_TYPE	ccusbdmx_eeprom_get(HANDLE h, LPVOID  buffer, USHORT size);

#endif // CCUSBDMX_H
