/*
 * DEMO.CPP - Demo code on how to use the ccusbdmx.dll with load-time 
 * dynamic linking. If the dll is not present this demo will not start.
 * Link you code against ccusbdmx.lib.
 *
 * This file is provided as is to allow an easy start with the
 * ccusbdmx driver and dll.
 *
 * In case of trouble please contact driver@lighting-solutions.de or
 * call +49/40/600877-51.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>

/* the ccusbdmx include file with all function definitions */
#include "ccusbdmx.h"

int main(int argc, char* argv[])
{
	HANDLE h;				/* handle to one interface */
	UCHAR  bufnew[0x201];	/* buffer for one dmx512 frame incl. startcode */
	USHORT version;			/* version number in BCD */
	USHORT slots;			/* number of received slots */
	USHORT timestamp;		/* timestamp [ms] returned from ccusbdmx_tx and rx */
	UCHAR  status;			/* status information from ccusbdmx_tx and rx */
	WCHAR  product[128];	/* product string read from device */
	WCHAR  serial[128];     /* serial number string read from device */
	CHAR   eeprom[128];		/* buffer with data to be read/written from/to eeprom */

	/* verify CCUSBDMX dll version */
	if (!CCUSBDMX_DLL_VERSION_CHECK())
	{
		printf("CCUSBDMX.DLL version does not match, giving up!\n");
		printf("found %i, expected %i\n", ccusbdmx_version(), CCUSBDMX_DLL_VERSION);
		getchar();
		return 1;
	}

	printf("Using CCUSBDMX.DLL version 0x%x\n\n", ccusbdmx_version());

    /* open the first devices (number 0) */
	if (!ccusbdmx_open(0, &h))
	{
		printf("no Cooper Controls usbdmx-interface available, giving up!\n");
		getchar();
		return 1;
	}

	/* read product and serial string and device version from device */
	if (!ccusbdmx_product_get(h, product, sizeof(product)))
	{
		printf("Error: reading product string failed\n");
		product[0] = 'L\0';
	}
	if (!ccusbdmx_serial_get(h, serial, sizeof(serial)))
	{
		printf("Error: reading serial string failed\n");
		product[0] = 'L\0';
	}
	if (!ccusbdmx_device_version(h, &version))
	{
		printf("Error: reading device version failed\n");
		version = 0;
	}

	/* identify the interface */
	printf("The interface found is a ");

	if      (ccusbdmx_is_oneuniverse(h))
		printf("LightFactory Zero88 one universe");
	else if (ccusbdmx_is_twouniverse(h))
		printf("LightFactory Zero88 2 universe");

	printf(" (%ws), Serial: %ws, Version: 0x%02x\n", product, serial, version);
	putchar('\n');

	/********************************************
	 * functions to access the dmx data buffers */

	/* send a frame: each time you call this function a new frame will be send out by the dongle */
	if (!ccusbdmx_tx(h,								/* handle to the interface */
					 0,								/* universe addressed
													   on DMX Dongle: only 0 is supported */
					 513,							/* number of slots to be transmitted, incl. startcode */
					 bufnew,						/* buffer with dmx data (bufnew[0] is the startcode */
					 CCUSBDMX_BULK_CONFIG_BLOCK,	/* configuration for this frame, see ccusbdmx.h for details */
					 100e-3f,						/* parameter for configuration [s], see uccsbdmx.h for details, 
													   in this case: block 100ms with respect to previous frame */
					 200e-6f,						/* length of break [s]. If 0, no break is generated */
					 20e-6f,						/* length of mark-after-break [s], If 0, no MaB is generated */
					 &timestamp,					/* timestamp of the frame [ms] */
					 &status))						/* status information */
		printf("ccusbdmx_tx(): error!\n");
	/* check transaction status, see ccusbdmx.h for details */
	if (!CCUSBDMX_BULK_STATUS_IS_OK(status))
		printf("ERROR: ccusbdmx_tx(): status = 0x%02x\n", status);

	/* receive a frame */
	if (!ccusbdmx_rx(h,				/* handle to the interface */
					 0,				/* universe addressed 
									   on DMX Dongle: only 0 is supported */
					 513,			/* number of slots to receive, incl. startcode */
					 bufnew,		/* buffer for the received data */
					 0.1f,			/* timeout to receive the total frame [s] */
					 1e-3f,			/* timeout to receive the next slot [s] */
					 &slots,		/* number of slots received */
					 &timestamp,	/* timestamp of the frame */
					 &status))		/* status information */
		printf("ccusbdmx_rx(): error!\n");
	/* check transaction status, see ccusbdmx.h for details */
	if (!CCUSBDMX_BULK_STATUS_IS_OK(status))
	{
		if (status == CCUSBDMX_BULK_STATUS_TIMEOUT)
			printf("INFO: ccusbdmx_rx() timed out, no frame received\n");
		else
			printf("ERROR: ccusbdmx_rx(): status = 0x%02x\n", status);
	}

	/********************************************************
	 * functions to read/write data to eeprom of the dongle */

	// remark: you can read/write any amount of data up to maximum value
	if (!ccusbdmx_eeprom_set(h, eeprom, sizeof(eeprom)))
	{
		printf("ERROR: writing to eeprom failed\n");
	}
	if (!ccusbdmx_eeprom_get(h, eeprom, sizeof(eeprom)))
	{
		printf("ERROR: reading from eeprom failed\n");
	}

	/* close the interface */
	ccusbdmx_close(h);

	printf("demo code finished\n");

	getchar();
	return 0;
}
